@@ -13,7 +13,7 @@ class TestClosedSocket(greentest.TestCase):
         sock.close()
         try:
             sock.send(b'a', timeout=1)
-            raise AssertionError("Should not get here")
+            self.fail("Should raise socket error")
         except (socket.error, OSError) as ex:
             if ex.args[0] != errno.EBADF:
                 if sys.platform.startswith('win'):
@@ -30,12 +30,13 @@ class TestRef(greentest.TestCase):
     switch_expected = False
 
     def test(self):
+        # pylint:disable=no-member
         sock = socket.socket()
-        assert sock.ref is True, sock.ref
+        self.assertTrue(sock.ref)
         sock.ref = False
-        assert sock.ref is False, sock.ref
-        assert sock._read_event.ref is False, sock.ref
-        assert sock._write_event.ref is False, sock.ref
+        self.assertFalse(sock.ref)
+        self.assertFalse(sock._read_event.ref)
+        self.assertFalse(sock._write_event.ref)
         sock.close()
 
 
