@@ -40,9 +40,21 @@ def _format_hub(hub):
         hub.__class__.__name__, id(hub), hub.thread_ident
     )
 
+
+def _get_thread_profile(_sys=sys):
+    if 'threading' in _sys.modules:
+        return _sys.modules['threading']._profile_hook
+
+
+def _get_thread_trace(_sys=sys):
+    if 'threading' in _sys.modules:
+        return _sys.modules['threading']._trace_hook
+
+
 class _WorkerGreenlet(RawGreenlet):
     # Exists to produce a more useful repr for worker pool
-    # threads/greenlets.
+    # threads/greenlets, and manage the communication of the worker
+    # thread with the threadpool.
 
     # Inform the gevent.util.GreenletTree that this should be
     # considered the root (for printing purposes)
@@ -57,6 +69,9 @@ class _WorkerGreenlet(RawGreenlet):
     # The hub of the threadpool we're working for. Just for info.
     _hub = None
 
+    # A cookie passed to task_queue.get()
+    _task_queue_cookie = None
+
     def __init__(self, threadpool):
         # Construct in the main thread (owner of the threadpool)
         # The parent greenlet and thread identifier will be set once the
@@ -76,8 +91,8 @@ class _WorkerGreenlet(RawGreenlet):
         self._stderr = stderr
         # We can capture the task_queue; even though it can change if the threadpool
         # is re-innitted, we won't be running in that case
-        self._task_queue = threadpool.task_queue
-
+        self._task_queue = threadpool.task_queue # type:gevent._threading.Queue
+        self._task_queue_cookie = self._task_queue.allocate_cookie()
         self._unregister_worker = threadpool._unregister_worker
 
         threadpool._register_worker(self)
@@ -131,13 +146,29 @@ class _WorkerGreenlet(RawGreenlet):
             name = co.co_name
             print('  File "%s", line %d, in %s' % (filename, lineno, name),
                   file=stderr)
+            tb = tb.tb_next
+
+    def _before_run_task(self, func, args, kwargs, thread_result,
+                         _sys=sys,
+                         _get_thread_profile=_get_thread_profile,
+                         _get_thread_trace=_get_thread_trace):
+        # pylint:disable=unused-argument
+        _sys.setprofile(_get_thread_profile())
+        _sys.settrace(_get_thread_trace())
+
+    def _after_run_task(self, func, args, kwargs, thread_result, _sys=sys):
+        # pylint:disable=unused-argument
+        _sys.setprofile(None)
+        _sys.settrace(None)
 
     def __run_task(self, func, args, kwargs, thread_result):
+        self._before_run_task(func, args, kwargs, thread_result)
         try:
             thread_result.set(func(*args, **kwargs))
         except: # pylint:disable=bare-except
             thread_result.handle_error((self, func), self._exc_info())
         finally:
+            self._after_run_task(func, args, kwargs, thread_result)
             del func, args, kwargs, thread_result
 
     def run(self):
@@ -146,20 +177,24 @@ class _WorkerGreenlet(RawGreenlet):
         exc_info = sys.exc_info
         fixup_hub_before_block = self.__fixup_hub_before_block
         task_queue_get = self._task_queue.get
+        task_queue_cookie = self._task_queue_cookie
         run_task = self.__run_task
         task_queue_done = self._task_queue.task_done
-        try:
+        try: # pylint:disable=too-many-nested-blocks
             while 1: # tiny bit faster than True on Py2
                 fixup_hub_before_block()
 
-                task = task_queue_get()
+                task = task_queue_get(task_queue_cookie)
                 try:
                     if task is None:
                         return
 
                     run_task(*task)
+                except:
+                    task = repr(task)
+                    raise
                 finally:
-                    task = None
+                    task = None if not isinstance(task, str) else task
                     task_queue_done()
         except Exception as e: # pylint:disable=broad-except
             print(
@@ -179,12 +214,13 @@ class _WorkerGreenlet(RawGreenlet):
             self._unregister_worker(self)
             self._unregister_worker = lambda _: None
             self._task_queue = None
+            self._task_queue_cookie = None
 
-            if hub_of_worker is not None:
-                hub_of_worker.destroy(True)
+        if hub_of_worker is not None:
+            hub_of_worker.destroy(True)
 
     def __repr__(self, _format_hub=_format_hub):
-        return "<ThreadPoolWorker at 0x%x thread_ident=0x%x hub=%s>" % (
+        return "<ThreadPoolWorker at 0x%x thread_ident=0x%x threadpool-hub=%s>" % (
             id(self),
             self._thread_ident,
             _format_hub(self._hub)
@@ -226,12 +262,23 @@ class ThreadPool(GroupMappingMixin):
     The `len` of instances of this class is the number of enqueued
     (unfinished) tasks.
 
+    Just before a task starts running in a worker thread,
+    the values of :func:`threading.setprofile` and :func:`threading.settrace`
+    are consulted. Any values there are installed in that thread for the duration
+    of the task (using :func:`sys.setprofile` and :func:`sys.settrace`, respectively).
+    (Because worker threads are long-lived and outlast any given task, this arrangement
+    lets the hook functions change between tasks, but does not let them see the
+    bookkeeping done by the worker thread itself.)
+
     .. caution:: Instances of this class are only true if they have
        unfinished tasks.
 
     .. versionchanged:: 1.5a3
        The undocumented ``apply_e`` function, deprecated since 1.1,
        was removed.
+    .. versionchanged:: 20.12.0
+       Install the profile and trace functions in the worker thread while
+       the worker thread is running the supplied task.
     """
 
     __slots__ = (
@@ -258,6 +305,8 @@ class ThreadPool(GroupMappingMixin):
         'task_queue',
     )
 
+    _WorkerGreenlet = _WorkerGreenlet
+
     def __init__(self, maxsize, hub=None):
         if hub is None:
             hub = get_hub()
@@ -411,7 +460,7 @@ class ThreadPool(GroupMappingMixin):
             self.fork_watcher.stop()
 
     def _adjust_wait(self):
-        delay = 0.0001
+        delay = self.hub.loop.approx_timer_resolution
         while True:
             self._adjust_step()
             if len(self._worker_greenlets) <= self._maxsize:
@@ -427,7 +476,7 @@ class ThreadPool(GroupMappingMixin):
             self.manager = Greenlet.spawn(self._adjust_wait)
 
     def _add_thread(self):
-        _WorkerGreenlet(self)
+        self._WorkerGreenlet(self)
 
     def spawn(self, func, *args, **kwargs):
         """
@@ -503,7 +552,7 @@ class _FakeAsync(object):
         pass
     close = stop = send
 
-    def __call_(self, result):
+    def __call__(self, result):
         "fake out for 'receiver'"
 
     def __bool__(self):
@@ -733,8 +782,9 @@ else:
                 future = self._threadpool.spawn(fn, *args, **kwargs)
                 return _FutureProxy(future)
 
-        def shutdown(self, wait=True):
-            super(ThreadPoolExecutor, self).shutdown(wait)
+        def shutdown(self, wait=True, **kwargs): # pylint:disable=arguments-differ
+            # In 3.9, this added ``cancel_futures=False``
+            super(ThreadPoolExecutor, self).shutdown(wait, **kwargs)
             # XXX: We don't implement wait properly
             kill = getattr(self._threadpool, 'kill', None)
             if kill: # pylint:disable=using-constant-test
